@@ -61,15 +61,17 @@ def getLogger(name=None, **kwargs):
                  name, ``__name__``.
     :type name: string
     """
-    if name not in _LOGGERS:
+    adapter = _LOGGERS.get(name)
+    if not adapter:
         # NOTE(jd) Keep using the `adapter' variable here because so it's not
         # collected by Python since _LOGGERS contains only a weakref
         adapter = KeywordArgumentAdapter(logging.getLogger(name), kwargs)
         _LOGGERS[name] = adapter
-    return _LOGGERS[name]
+    return adapter
 
 
-def setup(level=logging.WARNING, outputs=[output.STDERR], program_name=None):
+def setup(level=logging.WARNING, outputs=[output.STDERR], program_name=None,
+          capture_warnings=True):
     """Setup Python logging.
 
     This will setup basic handlers for Python logging.
@@ -77,12 +79,23 @@ def setup(level=logging.WARNING, outputs=[output.STDERR], program_name=None):
     :param level: Root log level.
     :param outputs: Iterable of outputs to log to.
     :param program_name: The name of the program. Auto-detected if not set.
+    :param capture_warnings: Capture warnings from the `warnings' module.
     """
-    # Sometimes logging occurs before logging is ready
-    # To avoid "No handlers could be found," temporarily log to sys.stderr.
     root_logger = logging.getLogger(None)
-    if not root_logger.handlers:
-        root_logger.addHandler(logging.StreamHandler())
+
+    # Remove all handlers
+    for handler in list(root_logger.handlers):
+        root_logger.removeHandler(handler)
+
+    # Add configured handlers
+    for out in outputs:
+        if isinstance(out, str):
+            out = output.preconfigured.get(out)
+            if out is None:
+                raise RuntimeError("Output {} is not available".format(out))
+        out.add_to_logger(root_logger)
+
+    root_logger.setLevel(level)
 
     program_logger = logging.getLogger(program_name)
 
@@ -92,15 +105,8 @@ def setup(level=logging.WARNING, outputs=[output.STDERR], program_name=None):
 
     sys.excepthook = logging_excepthook
 
-    # Remove all handlers
-    for handler in list(root_logger.handlers):
-        root_logger.removeHandler(handler)
-
-    # Add configured handlers
-    for o in outputs:
-        o.add_to_logger(root_logger)
-
-    root_logger.setLevel(level)
+    if capture_warnings:
+        logging.captureWarnings(True)
 
 
 def parse_and_set_default_log_levels(default_log_levels, separator='='):
